import Pool from '../src/index';

interface IArgs {
  name: string;
}

class Swimer {
  private _name: string;
  private _hobbits: string[] = ['swim'];
  private createdAt: Date;
  private _isHealth: boolean = true;

  constructor({ name }: IArgs) {
    this._name = name;
    this.createdAt = new Date();
  }

  public async say() {
    return `Hello! I'm ${this._name}`;
  }

  public async requestTel() {
    return '18888888888';
  }

  public async dead() {
    console.log(`I'm ${this._name}, I'm dead.`);
    this._isHealth = false;
  }

  public async destroy() {
    console.log(`${this._name} is destroying.`);
  }

  public inspect() {
    return `${this._name} has hobbits such as ${this._hobbits.join(' & ')}.`;
  }

  public isHealthy() {
    return this._isHealth;
  }
}

const pool = new Pool<IArgs, Swimer>({ Swimer, identity: args => args.name }, 'redis://localhost');
const pool2 = new Pool<IArgs, Swimer>({ Swimer, identity: args => args.name, channelPrefix: 'another-pool' });
const pool3 = new Pool<IArgs, Swimer>({ Swimer, identity: args => args.name }, 'redis://localhost:6379');
const pool4 = new Pool<IArgs, Swimer>({ Swimer, identity: args => args.name, channelPrefix: 'another-pool' });

describe('Pool', function() {
  this.timeout(100000);
  it('test', () => {
    pool.getSwimer({ name: 'A' }).then(swimer => {
      swimer.say().then(console.log);
    });
    pool.getSwimer({ name: 'B' }).then(swimer => {
      swimer.requestTel().then(console.log);
    });
    pool2.getSwimer({ name: 'A' }).then(swimer => {
      swimer.say().then(console.log);
    });
    pool2.getSwimer({ name: 'B' }).then(swimer => {
      swimer.say().then(console.log);
    });
    pool3.getSwimer({ name: 'A' }).then(swimer => {
      swimer.say().then(console.log);
    });
    setTimeout(() => {
      pool3.getSwimer({ name: 'B' }).then(swimer => {
        swimer.say().then(console.log);
      });
    }, 100);
    pool4.getSwimer({ name: 'A' }).then(swimer => {
      swimer.say().then(console.log);
    });
    pool4.getSwimer({ name: 'B' }).then(swimer => {
      swimer.say().then(console.log);
    });

    setTimeout(() => {
      pool.whereis({ name: 'A' }).then(console.log);
      pool.whereis({ name: 'B' }).then(console.log);
      pool2.whereis({ name: 'A' }).then(console.log);
      pool2.whereis({ name: 'B' }).then(console.log);
    }, 1800);

    setTimeout(() => {
      pool.state().then(state => console.log(JSON.stringify(state, null, 2)));
      pool2.state().then(console.log);
    }, 2000);
  });
});

describe('Destruct', function() {
  this.timeout(10000);
  it('should ok', async () => {
    const swimer = await pool.getSwimer({ name: 'C' });
    console.log(await swimer.say());
    console.log(await pool3.destruct({ name: 'C' }));
    console.log(await pool.whereis({ name: 'C' }));
  });
});

describe('ForceNewSwimer', function() {
  this.timeout(10000);
  it('should ok', async () => {
    const swimer = await pool.getSwimer({ name: 'C' });
    console.log(await swimer.say());
    console.log(await pool3.whereis({ name: 'C' }));
    const newSwimer = await pool3.forceGetSwimer({ name: 'C' });
    console.log(await newSwimer.say());
    console.log(await pool.state());
    console.log(await pool.state(pool3.id));
  });
});

describe('Call accept timeout', function() {
  this.timeout(10000);
  it('should ok', async () => {
    await pool3.forceGetSwimer({ name: 'C' });
    console.log(await pool.whereis({ name: 'C' }));
    const swimer = await pool.getSwimer({ name: 'C' });
    await pool3.destruct({ name: 'C' });
    console.log(await swimer.say());
    console.log(await pool.whereis({ name: 'C' }));
  });
});

describe('Inspect', function() {
  this.timeout(10000);
  it('should ok', async () => {
    await pool.getSwimer({ name: 'A' });
    console.log(await pool.inspect({ name: 'A' }));
  });
});

describe('Check healthy', function() {
  this.timeout(10000);
  it('should ok', async () => {
    const args = { name: 'B' };
    const swimer = await pool.getSwimer(args);
    console.log('Is he healthy?', await pool.isHealthy(args));
    await swimer.dead();
    console.log('Is he healthy?', await pool.isHealthy(args));
  });
});

after(() => {
  setTimeout(() => {
    pool.destroy();
    pool2.destroy();
    pool3.destroy();
    pool4.destroy();
  }, 3000);
});
